#include <iostream>
#include <string>
#include <map>
#include <stdexcept>
#include "Birthday.h"

using namespace std;

int main()
{
    map<string, Birthday> birthdays;

    birthdays["Ron"] = Birthday(1983, 7, 12);
    birthdays["Bob"] = Birthday(1997, 3, 25);
    birthdays["Sal"] = Birthday(1985, 8, 10);

    cout << "Some birthdays:" << endl;
    cout << "  Ron's birthday is " << birthdays["Ron"] << endl;
    cout << "  Jim's birthday is " << birthdays["Jim"] << endl;
    cout << "  Sal's birthday is " << birthdays["Sal"] << endl;

    try
    {
        cout << "  Bob's birthday is " << birthdays.at("Bob") << endl;
        cout << "  Tom's birthday is " << birthdays.at("Tom") << endl;
        cout << "  Sal's birthday is " << birthdays.at("Sal") << endl;
    }
    catch (const out_of_range& ex)
    {
        cout << endl << endl << "***** STL map out of range error: "
             << ex.what() << endl << endl;
    }

    cout << endl << "Inserting Ada's birthday:" << endl;
    auto adas_pair = pair<string, Birthday>("Ada", Birthday(1815, 12, 10));
    birthdays.insert(adas_pair);
    cout << "  Ada's birthday is " << birthdays["Ada"] << endl;

    cout << endl << "Inserting Ron's birthday again:" << endl;
    cout << "  Ron's birthday is " << birthdays["Ron"] << endl;

    Birthday rons_new_bd = Birthday(1900, 12, 18);
    auto rons_pair = pair<string, Birthday>("Ron", rons_new_bd);
    birthdays.insert(rons_pair);
    cout << "  Ron's birthday is " << birthdays["Ron"] << endl;

    cout << endl << "Iterating over the birthdays:" << endl;
    for (auto it = birthdays.begin(); it != birthdays.end(); it++)
    {
        cout << "  " << it->first << ": " << it->second << endl;
    }
    cout << "There are " << birthdays.size() << " birthdays." << endl;

    cout << endl << "Erase Bob's birthday:" << endl;
    auto it = birthdays.find("Bob");
    birthdays.erase(it);

    if (birthdays.find("Bob") == birthdays.end())
    {
        cout << "  Bob's birthday is now unknown." << endl;
    }
    else
    {
        cout << "  Bob's birthday is " << birthdays["Bob"] << endl;
    }

    cout << endl << "How many birthday entries:" << endl;
    cout << "  Ron: " << birthdays.count("Ron") << endl;
    cout << "  Bob: " << birthdays.count("Bob") << endl;

    cout << endl << "Use for_each with a lambda expression:" << endl;
    for_each(birthdays.begin(), birthdays.end(),
             [] (pair<string, Birthday> p)
             {
                 cout << "  " << p.first << ": " << p.second << endl;
             });

    cout << endl << "Done!" << endl;
    return 0;
}
